#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-03-31
#
# Monitors Cisco WLC power supplys 
#
# 2016-03-31: initial release
# 2021-07-15: rewritten for CMK 2.0
#

from typing import Dict, Optional
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    SNMPTree,
    contains,
    State,
)

_ps_state = {
    '0': False,
    '1': True,
}


def parse_cisco_wlc_power(string_table: StringTable) -> Optional[Dict[str, bool]]:
    try:
        ps1, ps1op, ps2, ps2op = string_table[0]
    except IndexError:
        return

    ps = {}
    if _ps_state.get(ps1):
        ps.update({'1': _ps_state.get(ps1op)})
    if _ps_state.get(ps2):
        ps.update({'2': _ps_state.get(ps2op)})
    return ps


def discovery_cisco_wlc_power(section: Dict[str, bool]) -> DiscoveryResult:
    for key in section:
        yield Service(item=f'{key}')


def check_cisco_wlc_power(item, params, section: Dict[str, bool]) -> CheckResult:
    try:
        if section[item]:
            yield Result(state=State.OK, summary=f'Power supply {item}')
        else:
            yield Result(state=State(params.get('not_operational', 2)),
                         summary=f'Power supply {item} is non operational')
    except KeyError:
        yield Result(state=State(params.get('not_present', 2)), summary=f'Power supply {item} is not present')
        return


register.snmp_section(
    name='cisco_wlc_power',
    parse_function=parse_cisco_wlc_power,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.14179.1.1.3',  # AIRESPACE-SWITCHING-MIB::agentSwitchInfoGroup
        oids=[
            '2',  # agentSwitchInfoPowerSupply1Present
            '3',  # agentSwitchInfoPowerSupply1Operational
            '4',  # agentSwitchInfoPowerSupply2Present
            '5',  # agentSwitchInfoPowerSupply2Operational
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'Cisco Controller'),  # sysDescr
)

register.check_plugin(
    name='cisco_wlc_power',
    service_name='Power Supply %s',
    discovery_function=discovery_cisco_wlc_power,
    check_function=check_cisco_wlc_power,
    check_default_parameters={},
    check_ruleset_name='cisco_wlc_power',
)
